package CPANPLUS::Shell::Default::Plugins::PacmanPlugin;

use strict;
use File::Spec;
use CPANPLUS::Error;
use Locale::Maketext::Simple    Class => 'CPANPLUS', Style => 'gettext';

use CPANPLUS::Dist::Pacman qw[archpkgname make_abscache %perl_abs %ARCHPKGCONF];

sub plugins { ( makepkg => 'makepkg' ) } 

sub makepkg {
    my $class   = shift;
    my $shell   = shift;
    my $cb      = shift;
    my $cmd     = shift;
    my $name   = shift || '';
    my $opts    = shift || {};
	my $conf = $cb->configure_object;
	
    my $obj = $cb->parse_module( module => $name ) or (
                die "Cannot make a module object out of $name" );
	
	my $clean           = $opts->{'clean'} 
						? $opts->{'clean'} 
						: $ARCHPKGCONF{'clean'} ;
	$ARCHPKGCONF{'clean'} = $clean;

	my $localbasedir=  exists $opts->{'localbasedir'} 
							? $opts->{'localbasedir'} 
							: $ARCHPKGCONF{'localbasedir'};
	$ARCHPKGCONF{'localbasedir'} = $localbasedir;

	my $build     =  exists $opts->{'build'} 
							? $opts->{'build'} 
							: $ARCHPKGCONF{'makepkg'};
	$ARCHPKGCONF{'makepkg'} = $build;

	my $makepkgflags     =  exists $opts->{'makepkgflags'} 
							? $opts->{'makepkgflags'} 
							: $ARCHPKGCONF{'makepkgflags'};
	$ARCHPKGCONF{'makepkgflags'} = $makepkgflags;

    my $force       = exists $opts->{'force'}      
                        ? $opts->{'force'}   
                        : $conf->get_conf('force');
	$conf->set_conf( force => $force );

	$conf->set_conf( dist_type  => 'CPANPLUS::Dist::Pacman' ); 

	
    my $pkgname = $obj->package_name;

    if (exists $perl_abs{$pkgname} and ! $force) {
       my $archpak = $perl_abs{$pkgname}{arch_package};
       print "The package $archpak can be installed from Arch Linux repositories!\n";
       exit 0;
    };
   
    my $archpkgname = &archpkgname($pkgname);
     
    my $makepkgdir = File::Spec->catfile($localbasedir, $archpkgname);
    my $extractdir = File::Spec->catfile($makepkgdir, 'src');
    mkdir($makepkgdir) unless -d $makepkgdir;
    mkdir($extractdir) unless -d $extractdir;
    my $targets = $build ? 'create' 
             : 'prepare' ;  

    my $dist = eval { $obj->install(
                                extractdir      => $extractdir,
                                target          => $targets
                            )
                }; 

    if ( $@ ) {
        warn loc("Dist creation of '%1' failed: '%2'", $obj->module, $@);
        next;
    } ;       

    ### we didn't get a dist object back?
    $dist or (warn loc("Unable to create '%1' dist of '%2'",
                        'Pacman', $obj->module), 
              next);
    my $created = $obj->status->created;
    my $finalmsg = $created ? "Package $archpkgname built successfully.\nYou may now install it with pacman"
        : "PKGBUILD for package $archpkgname generated successfully";
        
    print $finalmsg, "\n";
}

sub makepkg_help {
	return loc(
     "  /makepkg Some::Module # Make a Pacman package for Some::Module\n" .
     "  Options (defaults and other options from cpan4pacman.yml):	  \n" .
     "   --[no]build          # Whether or not to build after generating the PKGBUILD\n" .
     "   --[no]clean          # Pass option --clean to makepkg\n" .
     "   --[no]force          # Forces most operations\n" .
     "   --[no]makefile       # Prefer Makefile.PL over Build.PL\n" .
     "   --[no]maketest       # Run 'make test'\n" .
     "   --makepkgflags=s     # Options to pass to makepkg" );
}

1;
